#!/bin/sh
if [ "$#" -lt 1 ]; then
	exec >&2
	echo "usage: `basename $0` user group uid gid homedir"
	exit 1
fi

TUSER="$1"
TGROUP="${2:-${TUSER}}"
TUID="$3"
TGID="${4:-${TUID}}"
THOME="${5:-/nonexistent}"
TSHELL="/sbin/nologin"

OSTYPE=`uname -s | tr A-Z a-z`
case "$OSTYPE" in
*bsd|bsdi)
	if pw groupshow ${TGROUP} >/dev/null 2>&1; then
		echo "===> Group ${TGROUP} already exists, skipping"
	else
		if pw groupadd ${TGROUP} ${TGID:+-g${TGID}}; then
			echo "===> Added group ${TGROUP}"
		else
			echo "===> Adding group ${TGROUP} failed"
			exit 1
		fi
	fi

	if pw usershow "${TUSER}" >/dev/null 2>&1; then
		echo "===> User ${TUSER} already exists, skipping"
	else
		if pw useradd ${TUSER} ${TUID:+-u${TUID}} -g ${TGROUP} -h - \
			-s "$TSHELL" -d "$THOME" \
			-c 'trafd daemon user'
		then
			echo "===> Added user ${TUSER}"
		else
			echo "===> Adding user ${TUSER} failed"
		fi
	fi
	;;
linux)
	if grep -q "^${TGROUP}:" /etc/group; then
		echo "===> Group ${TGROUP} already exists, skipping"
	else
		if groupadd -r ${TGID:+-g${TGID}} ${TGROUP}; then
			echo "===> Added group ${TGROUP}"
		else
			echo "===> Adding group ${TGROUP} failed"
			exit 1
		fi
	fi

	if grep -q "^${TUSER}:" /etc/passwd; then
		echo "===> User ${TUSER} already exists, skipping"
	else
		if useradd -r ${TUID:+-u${TUID}} -g ${TGROUP} \
			-s "$TSHELL" -d "$THOME" \
			-c 'trafd daemon user' -M ${TUSER}
		then
			echo "===> Added user ${TUSER}"
		else
			echo "===> Adding user ${TUSER} failed"
		fi
	fi
	;;
esac
