/*
 *	Copyright (c) 1999 RISS-Telecom Networking Center
 *
 *	Copyright (c) 1993 The CAD lab of the
 *	Novosibirsk Institute of Broadcasting and Telecommunication
 *
 *	BPFT $Id: traffic.h,v 1.3 2006/09/27 05:00:32 frol Exp $
 *
 *	$Log: traffic.h,v $
 *	Revision 1.3  2006/09/27 05:00:32  frol
 *	 - Fixes for 64bit platforms (still need to fix usage of time_t in trafdb);
 *	 - Add trafdb and trafrev into the build;
 *	 - Add NO_MAN Makefile defines for FreeBSD >= 5.x, leave NOMAN compat;
 *	 - Fix trafrev to not seek beyond beginning of the file.
 *	
 *	Revision 1.1.1.1  2000/04/07 05:26:48  count
 *	Initial revision
 *	
 * Revision 1.4  1994/01/14  11:20:23  bob
 * Little game with MAX params
 *
 * Revision 1.3  1994/01/06  14:54:00  bob
 * Modifications for new hash table format, removed any restriction.
 *
 * Revision 1.2  1993/12/05  13:45:24  bob
 * Increased limit of bytes up to MTU on loopback
 *
 * Revision 1.1  1993/10/20  16:03:10  bob
 * Initial revision
 *
 *
 * Redistribution and use in source forms, with and without modification,
 * are permitted provided that this entire comment appears intact.
 * Redistribution in binary form may occur without any restrictions.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' WITHOUT ANY WARRANTIES OF ANY KIND.
 */

#ifndef _TRAFD_TRAFFIC_H
#define _TRAFD_TRAFFIC_H

#include <sys/types.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>

/*	File with major definitions	*/

/*
 * Maximum numder of the stored data records for autobackup to disk.
 */
#ifndef MAX_TO_SAVE
#define MAX_TO_SAVE		104800
#endif

/*
 * Maximum size of the hashing memory table.
 */
#ifndef MAX_T_ENTRIES
#define	MAX_T_ENTRIES		(int)(MAX_TO_SAVE*1.3)
#endif

/*
 * Magic number, this is offset used in hash algoritm to moved on
 * hash table. Do not edit, this is the nice numder.
 */
#ifndef E_OFFSET
#define	E_OFFSET		1	/* XXX */
#endif

/*
 * This is the main hash function definition, produce start index
 * for begin hash table scan procedure. Very scientific formula. :)
 */
/*
 * But look at _hashfunc1()! ;)
 */
#define HASHFUNC_OLD(p) ((p)^((p)>>7)^((p)>>15)^((p)>>23))

#ifdef WITH_OLD_HASH
#define hash(e) _hashfunc_old(e)
#else
#define hash(e) _hashfunc1(e)
#endif

/* Save time difference between struct timeval's tv2 and tv1 in tv0 */
#define TIMEDIFF(tv1, tv2, tv0) do { \
		(tv0)->tv_usec = (tv2)->tv_usec - (tv1)->tv_usec; \
		(tv0)->tv_sec = (tv2)->tv_sec - (tv1)->tv_sec; \
		if ((tv0)->tv_usec < 0) \
			(tv0)->tv_sec--, (tv0)->tv_usec += 1000000L; \
	} while (0)

/*
 * Default minimum record size. Records with less number of
 * bytes will not be saved or piped to trafstat.
 *
 * Note, that in trafd prior to 3.0.3-b5 this was set to 2048
 * which caused it to drop traffic records less than 2048 bytes.
 */
#ifndef MIN_RECORD_SIZE
#define MIN_RECORD_SIZE 0
#endif

/* struct timeval but with fixed 32bit values */
struct tv32 {
	int32_t tv32_sec;
	int32_t tv32_usec;
};

#define	TV2TV32(tv, t32)	do { (t32)->tv32_sec = (int32_t)((tv)->tv_sec); \
	(t32)->tv32_usec = (int32_t)((tv)->tv_usec); } while(0)
#define	TV322TV(t32, tv)	do { (tv)->tv_sec = (time_t)((t32)->tv32_sec); \
	(tv)->tv_usec = (suseconds_t)((t32)->tv32_usec); } while(0)
#define TV32CLEAR(t32)		((t32)->tv32_sec = (t32)->tv32_usec = 0)
#define	TV32CMP(t1, t2, cmp)	(((t1)->tv32_sec == (t2)->tv32_sec) ? \
		((t1)->tv32_usec cmp (t2)->tv32_usec) : \
		((t1)->tv32_sec cmp (t2)->tv32_sec))
#define	TV32SET(t, f)		do { (t)->tv32_sec = (f)->tv32_sec; \
		(t)->tv32_usec = (f)->tv32_usec; } while (0)

/*
 * Single record's format description.
 */
struct t_entry {
	struct in_addr	in_ip, out_ip;	/* src ip addr and dst ip addr */
	uint8_t	ip_protocol;	/* which protocol been used (/etc/protocols) */
	uint8_t	who_srv;	/* who was server flag */
	uint16_t	p_port;		/* which port been used (/etc/services) */
	uint32_t	n_psize;	/* how many bytes in ip datagrams passed */
	uint32_t	n_bytes;	/* how many data bytes passed */
};
typedef struct t_entry *p_entry;

/* data file record header */
struct t_header {
	uint32_t    t_size;
	struct tv32 start;
	struct tv32 stop;
};
typedef struct t_header *p_header;

struct t_record {
	struct t_header	header;
	p_entry		entry;
};
typedef struct t_record *p_record;

/* record matching pattern */
struct pat_mask {
	struct  in_addr in_ip, out_ip;
	struct  in_addr in_mask, out_mask;
	uint8_t	ip_protocol;
	uint16_t	p_port;
};

#ifndef	IPPROTO_OSPF
#define	IPPROTO_OSPF	89
#endif
#ifndef	IPPROTO_IGMP
#define	IPPROTO_IGMP	2
#endif

#ifndef IPPORT_RESERVED
#define IPPORT_RESERVED	1024
#endif
#ifndef IPPORT_USERRESERVED
#define IPPORT_USERRESERVED	5000
#endif
#ifndef IPPORT_HIFIRSTAUTO
#define IPPORT_HIFIRSTAUTO	49152
#endif
#ifndef IPPORT_HILASTAUTO
#define IPPORT_HILASTAUTO	65535
#endif

/*
int traf_init(int);
int insertentry(p_entry);
void traf_add(p_entry);
void traf_tcp(struct tcphdr *, struct ip *);
void traf_udp(struct udphdr *, struct ip *);
void traf_ip(struct ip *);
void traf_save(char *, char *);
void traf_restore();
void traf_clear();
int traf_send(int);
void onpipe();
void processing_ip(const uint8_t *, u_int);
*/

#endif /* _TRAFD_TRAFFIC_H */
