/*
 * Copyright (c) 1999-2002
 * 	RISS-Telecom Networking Center
 *
 * Copyright (c) 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @(#) $Header: /cvsroot/trafd/trafd/include/util.h,v 1.3 2008/02/21 07:20:29 lorf Exp $ (LBL)
 */

#ifndef __trafd_util_h
#define __trafd_util_h

#include <limits.h>
#include <inttypes.h>

#include "gnuc.h"

#ifndef min
#define min(a,b) ((a)>(b)?(b):(a))
#endif
#ifndef max
#define max(a,b) ((b)>(a)?(b):(a))
#endif

#ifndef NTOHL
#define NTOHL(x)	(x) = ntohl(x)
#define NTOHS(x)	(x) = ntohs(x)
#define HTONL(x)	(x) = htonl(x)
#define HTONS(x)	(x) = htons(x)
#endif

/*
 * Format to print 64 bit integers.
 * PRIu64 is a C99 standard, define it ourselvs to be backward compatible.
 */
#if defined(LONG_BIT)
# if LONG_BIT == 64
#  define PRINTF_U64	"lu"
#  define PRINTF_I64	"ld"
# elif LONG_BIT == 32
#  define PRINTF_U64	"llu"
#  define PRINTF_I64	"lld"
# endif
#elif defined(__WORDSIZE)
# if __WORDSIZE == 64
#  define PRINTF_U64	"lu"
#  define PRINTF_I64	"ld"
# elif __WORDSIZE == 32
#  define PRINTF_U64	"llu"
#  define PRINTF_I64	"lld"
# endif
#endif

#ifndef PATH_MAX
#define PATH_MAX	256
#endif

/* strlcpy from pcap-int.h */
#ifndef HAVE_STRLCPY
#define strlcpy(x, y, z) \
	(strncpy((x), (y), (z)), \
	 ((z) <= 0 ? 0 : ((x)[(z) - 1] = '\0')), \
	 strlen((y)))
#endif

extern char *program_name;	/* used to generate self-identifying messages */

extern void set_program_name(char *);
#if __STDC__
extern void error(const char *, ...)
    __attribute__((format (printf, 1, 2)));
extern void errorx(const char *, ...)
    __attribute__((format (printf, 1, 2)));
extern void warning(const char *, ...)
	__attribute__ ((format (printf, 1, 2)));
extern void warningx(const char *, ...)
	__attribute__ ((format (printf, 1, 2)));
#endif
extern char *copy_argv(char **);
extern char *read_infile(char *);
extern char *stripdir(char *);
extern uint32_t net_mask(uint32_t *);
extern void setcred(const char *, const char *);
extern int writepid(char *file, pid_t pid);
extern pid_t readpid(char *file);

#endif /* __trafd_util_h */
