/*
 * Copyright (c) 2002 RISS-Telecom, Novosibirsk, Russia
 * $RISS: trafd/lib/trafdb.c,v 1.2 2006/09/27 05:00:32 frol Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/param.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pcap.h>

#include "interface.h"
#include "addrtoname.h"
#include "traffic.h"
#include "util.h"

int traf_write(FILE *, FILE *, struct t_header *);

static int no = 0;

int
get_header(FILE *fd, p_record rec)
{
	if (fread((p_header)&rec->header, sizeof(struct t_header), 1, fd) != 1) {
		if (!feof(fd))
			warning("fread: can't read header");
		return 0;
	}
	if (!rec->header.t_size)
		warningx("empty table");
	return 1;
}

int
get_entry(FILE *fd, p_record r)
{
	int retval;

	if (r->header.t_size) {
		r->entry = (p_entry) malloc(r->header.t_size * sizeof(struct t_entry));
		if (r->entry == NULL) {
			warning("Can't allocate memory for %d entris", r->header.t_size);
			return 1;
		}

		retval = fread(r->entry, r->header.t_size * sizeof(struct t_entry), 1, fd);
		if (retval != 1) {
			free(r->entry);
			warning("fread: can't read table");
			return 1;
		}
	}
	else
		r->entry = NULL;

	return 0;
}

void
log_loop(FILE *fd, FILE *fdw, struct timeval tb, struct timeval te)
{
	struct t_record r;
	off_t last_pos = 0;
	int got_it = 0, is_num;
	struct tv32 temp_start, temp_end;

	is_num = (tb.tv32_usec || te.tv32_usec)? 1 : 0;
	TV32CLEAR(&temp_start);
	TV32CLEAR(&temp_end);

	if (fdw == NULL)
		fdw = stdout;

	while (get_header(fd, &r)) {
		last_pos = ftello(fd);
		no++;

		if(is_num) {
			temp_start.tv32_usec = temp_end.tv32_usec = no;
		} else {
			TV32SET(&temp_start, &r.header.start);
			TV32SET(&temp_end, &r.header.stop);
		}

		if (TV32CMP(&temp_start, &tb, >=) && timercmp(&temp_end, &te, <=)) {
			got_it++;
			if (get_entry(fd, &r) != 0)
				return;
			if(traf_write(fd, fdw, &r) != 0)
				return;
		} else
			fseeko(fd, h.t_size * sizeof(struct t_entry), SEEK_CUR);
	}

	if (!got_it && last_pos) {
		fseeko(fd, last_pos - sizeof(h), SEEK_SET);
		get_header(fd, &h);
		traf_write(fd, fdw, &h);
	}

	return;
}

int
traf_write(FILE *fd, FILE *fdw, p_header h)
{
	struct t_entry *block = NULL;
	int retval;

	retval = !fwrite(h, sizeof(struct t_header), 1, fdw);

	if (retval)
		error("fwrite header");

	if (h->t_size) {
		block = (struct t_entry *) malloc(h->t_size * sizeof(struct t_entry));
		if (block == NULL) {
			warning("malloc");
			return 1;
		}

		retval = fread(block, sizeof(struct t_entry), h->t_size, fd);
		if (retval != h->t_size) {
			free(block);
			warning("fread: can't read table");
			return 1;
		}

		retval = !fwrite(block, sizeof(struct t_entry), h->t_size, fdw);

		free(block);

		if (retval)
			error("fwrite table");
	}

	return 0;
}

