/*
 * Copyright (c) 1999
 * 	RISS-Telecom Networking Center
 * 
 * Copyright (c) 1990, 1991, 1993, 1994, 1995, 1996, 1997
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>

#include <ctype.h>
#include <errno.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <pcap.h>
#include <stdio.h>
#if __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>

#include "gnuc.h"

#include "util.h"

char *program_name = NULL;

void
set_program_name(char *argv0)
{
	char *cp;

	if ((cp = strrchr(argv0, '/')) != NULL)
		program_name = cp + 1;
	else
		program_name = argv0;
}

/* VARARGS */
void
#if __STDC__
error(const char *fmt, ...)
#else
error(fmt, va_alist)
	const char *fmt;
	va_dcl
#endif
{
	int err_no;
	va_list ap;

	err_no = errno;
	if (program_name)
		(void)fprintf(stderr, "%s: ", program_name);
#if __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif
	if (fmt && *fmt) {
		(void)vfprintf(stderr, fmt, ap);
		(void)fprintf(stderr, ": ");
	}
	va_end(ap);
	(void)fprintf(stderr, "%s\n", strerror(err_no));
	exit(1);
	/* NOTREACHED */
}

/* VARARGS */
void
#if __STDC__
errorx(const char *fmt, ...)
#else
errorx(fmt, va_alist)
	const char *fmt;
	va_dcl
#endif
{
	va_list ap;

	if (program_name)
		(void)fprintf(stderr, "%s: ", program_name);
#if __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif
	if (fmt && *fmt) {
		(void)vfprintf(stderr, fmt, ap);
		if (fmt[strlen(fmt)-1] != '\n')
			(void)fprintf(stderr, "\n");
	}
	va_end(ap);
	exit(1);
	/* NOTREACHED */
}

/* VARARGS */
void
#if __STDC__
warning(const char *fmt, ...)
#else
warning(fmt, va_alist)
	const char *fmt;
	va_dcl
#endif
{
	int err_no;
	va_list ap;

	err_no = errno;
	if (program_name)
		(void)fprintf(stderr, "%s: WARNING: ", program_name);
#if __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif
	if (fmt && *fmt) {
		(void)vfprintf(stderr, fmt, ap);
		(void)fprintf(stderr, ": ");
	}
	va_end(ap);
	(void)fprintf(stderr, "%s\n", strerror(err_no));
}

/* VARARGS */
void
#if __STDC__
warningx(const char *fmt, ...)
#else
warningx(fmt, va_alist)
	const char *fmt;
	va_dcl
#endif
{
	va_list ap;

	if (program_name)
		(void)fprintf(stderr, "%s: WARNING: ", program_name);
#if __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif
	if (fmt && *fmt) {
		(void)vfprintf(stderr, fmt, ap);
		if (fmt[strlen(fmt)-1] != '\n')
			(void)fprintf(stderr, "\n");
	}
	va_end(ap);
}

/*
 * Copy arg vector into a new buffer, concatenating arguments with spaces.
 */
char *
copy_argv(char **argv)
{
	char **p;
	u_int len = 0;
	char *buf;
	char *src, *dst;

	p = argv;
	if (*p == 0)
		return 0;

	while (*p)
		len += strlen(*p++) + 1;

	buf = (char *)malloc(len);
	if (buf == NULL)
		error("copy_argv: malloc");

	p = argv;
	dst = buf;
	while ((src = *p++) != NULL) {
		while ((*dst++ = *src++) != '\0')
			;
		dst[-1] = ' ';
	}
	dst[-1] = '\0';

	return buf;
}

char *
read_infile(char *fname)
{
	int fd, cc;
	char *cp;
	struct stat buf;

	fd = open(fname, O_RDONLY);
	if (fd < 0)
		error("can't open %s for reading", fname);

	if (fstat(fd, &buf) < 0)
		error("can't stat %s", fname);

	cp = malloc((u_int)buf.st_size + 1);
	cc = read(fd, cp, (int)buf.st_size);
	if (cc < 0)
		error("read %s", fname);
	if (cc != buf.st_size)
		errorx("short read %s (%d != %d)", fname, cc, (int)buf.st_size);
	cp[(int)buf.st_size] = '\0';

	return (cp);
}

char *
stripdir(char *s) {
	char *cp;

	cp = rindex(s, '/');
	return (cp != 0) ? cp + 1: s;
}

uint32_t
net_mask(uint32_t *addr) {
	uint32_t m = 0xffffffff;

	if (*addr)
		while ((*addr & 0xff000000) == 0)
			*addr <<= 8, m <<=8;

	return m;
}

/* write pidfile */
int
writepid(char *file, pid_t pid)
{
	FILE *fp;
	int fd;

	fd = open(file, O_CREAT|O_EXCL|O_WRONLY, 0600);
	if (fd == -1)
		return -1;
	fp = fdopen(fd, "wb");
	if (fp == NULL)
		return -1;
	(void)fprintf(fp,"%d\n", pid);
	fclose(fp);
	return 0;
}

pid_t
readpid(char *file)
{
	FILE *f;
	char buf[32], *end;
	pid_t pid;

	if ((f = fopen(file, "r")) == NULL)
		error("open %s for reading", file);
	if (fgets(buf, sizeof(buf), f) == NULL)
		error("reading %s", file);
	pid = (pid_t)strtol(buf, &end, 10);
	if (end == buf)
		return 0;
	return pid;
}
