/*
 *	Copyright (c) 1999 RISS-Teleocm Networking Center
 *
 *	Copyright (c) 1993 The CAD lab of the
 *	Novosibirsk Institute of Broadcasting and Telecommunication
 *
 *	BPFT $Id: traffic.c,v 1.2 2006/09/27 05:00:32 frol Exp $
 *
 *	$Log: traffic.c,v $
 *	Revision 1.2  2006/09/27 05:00:32  frol
 *	 - Fixes for 64bit platforms (still need to fix usage of time_t in trafdb);
 *	 - Add trafdb and trafrev into the build;
 *	 - Add NO_MAN Makefile defines for FreeBSD >= 5.x, leave NOMAN compat;
 *	 - Fix trafrev to not seek beyond beginning of the file.
 *	
 *	Revision 1.1.1.1  2000/04/07 05:26:48  count
 *	Initial revision
 *	
 * Revision 1.4  1994/01/14  11:50:38  bob
 * Now really logged tcp/udp fragmented datagramms, little syslog's corrections
 *
 * Revision 1.3  1994/01/06  15:00:41  bob
 * Modification for new hash table format, optimized for speed
 *
 * Revision 1.2  1993/11/01  15:03:22  bob
 * Improve interrupts queue processing
 *
 * Revision 1.1  1993/10/20  16:04:41  bob
 * Initial revision
 *
 *
 * Redistribution and use in source forms, with and without modification,
 * are permitted provided that this entire comment appears intact.
 * Redistribution in binary form may occur without any restrictions.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' WITHOUT ANY WARRANTIES OF ANY KIND.
 */

/*	traffic.c - collect network traffic	*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <syslog.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
/*#include <netinet/ip_var.h>*/
#include <netinet/udp.h>
/*#include <netinet/udp_var.h>*/
#include <netinet/tcp.h>
/*#include <netinet/tcpip.h>*/
#include <netdb.h>
#include <pcap.h>
#include <limits.h>

#include "interface.h"
#include "traffic.h"
#include "util.h"

#include "trafd.h"

#ifdef WITH_HASH_TESTS
extern int64_t hash_chain_len, hash_chain_n;
extern int32_t hash_chain_max;
#endif

extern u_int save_min_record_size, pipe_min_record_size;

static int iplen;
static int dlen;
static struct tv32 begin_time;
static u_char s_port_big[IPPORT_RESERVED];

#define CHECK_SRV(p, proto) (s_port_big[(p) & (IPPORT_RESERVED-1)] & (proto))
#if IPPORT_HILASTAUTO < USHRT_MAX
#define IS_USER_PORT(p) (((p) >= IPPORT_RESERVED && (p) < IPPORT_USERRESERVED) || \
	((p) >= IPPORT_HIFIRSTAUTO && \
	(p) <= IPPORT_HILASTAUTO))
#else
/* (p) <= IPPORT_HILASTAUTO is always true because (p) is an unsigned short */
#define IS_USER_PORT(p) (((p) >= IPPORT_RESERVED && (p) < IPPORT_USERRESERVED) || \
	((p) >= IPPORT_HIFIRSTAUTO))
#endif

struct t_entry *entries;
/*
 * Definition of the global counters.
 */
int32_t n_entry = 0;	/* counter for number of records */

/*
 * Initialize traffic collector.
 */
int
traf_init(restore)
	int restore;
{
	register int i;
	struct servent *sv;
	struct timeval tv;

	entries = (struct t_entry *) calloc(MAX_T_ENTRIES, sizeof(struct t_entry));
	if (entries == NULL)
		return -1;
	(void) memset(s_port_big, 0, sizeof(s_port_big));
	setservent(1);
	while ((sv = getservent())) {
		NTOHS(sv->s_port);
		if (sv->s_port >= IPPORT_RESERVED) {
			i = sv->s_port & (IPPORT_RESERVED-1);
			if (strcmp(sv->s_proto, "tcp") == 0)
				s_port_big[i] |= IPPROTO_TCP;
			else if (strcmp(sv->s_proto, "udp") == 0)
				s_port_big[i] |= IPPROTO_UDP;
		}
	}
	endservent();
	n_entry = 0;
	if (restore)
		traf_restore();
	else {
		gettimeofday(&tv, (struct timezone *)NULL);
		TV2TV32(&tv, &begin_time);
	}
	return 0;
}

/*
 * Hash by in_ip.s_addr.
 */
static inline u_int
_hashfunc_old(void *e)
{
	return HASHFUNC_OLD(((p_entry)e)->in_ip.s_addr);
}

/*
 * With this hash function trafd can be more than 30 times faster
 * than with _hashfunc_old().
 */
static inline u_int
_hashfunc1(void *e)
{
	u_int *i = (e);
	return (i[0] ^ i[1] ^ i[2]);
}

/*
 * Insert entry into the hash table.
 */
int
insertentry( e )
	p_entry e;
/* return -1 if not found or -2 if not found & table full */
{
	register unsigned int inx;
	unsigned int oldinx;
	int ret;
#ifdef WITH_HASH_TESTS
	int chain = 0;
#endif

	oldinx = inx = hash(e) % MAX_T_ENTRIES;
	do {
		/* if empty slot found */
		if (!entries[inx].n_psize) {
			entries[inx] = *e;
			ret = -1;
			goto done;
		}
		if (e->in_ip.s_addr == entries[inx].in_ip.s_addr &&
		    e->out_ip.s_addr == entries[inx].out_ip.s_addr &&
		    e->ip_protocol == entries[inx].ip_protocol &&
		    e->who_srv == entries[inx].who_srv &&
		    e->p_port == entries[inx].p_port)
		{
			ret = (int)inx;
			goto done;
		}
		inx = (inx + E_OFFSET) % MAX_T_ENTRIES;
#ifdef WITH_HASH_TESTS
		chain++;
#endif
	} while (inx != oldinx);
	ret = -2;

done:
#ifdef WITH_HASH_TESTS
	hash_chain_len += chain;
	hash_chain_n++;
	if (chain > hash_chain_max)
		hash_chain_max = chain;
#endif
	return ret;
}

/*
 * Add new record to hash table or modify existed record.
 */
void
traf_add(p_entry pte)
{
	int t_idx;

	t_idx = insertentry(pte);

	if (n_entry > MAX_TO_SAVE || t_idx == -2) {
		(void)syslog(LOG_WARNING, "(%s) table full, autobackup",
			     device_name);
		if (fork() == 0) {
			(void)traf_save(file_backup, "a");
			exit(0);
		}
		(void)traf_clear();
		return;
	}
	if (t_idx >= 0)
		entries[t_idx].n_bytes += pte->n_bytes,
		entries[t_idx].n_psize += pte->n_psize;
	else
		n_entry++;
	return;
}

/*
 * Calculate data length and server port in tcp packets.
 */
void
traf_tcp(tp, ip)
	struct tcphdr *tp;
	struct ip *ip;
{
	static struct t_entry t;

	if ((u_char *)(tp + 1) > snapend || dlen < sizeof(struct tcphdr))
		return;

	NTOHS(tp->th_sport);
	NTOHS(tp->th_dport);

	t.in_ip.s_addr = ip->ip_src.s_addr;
	t.out_ip.s_addr = ip->ip_dst.s_addr;
	t.ip_protocol = IPPROTO_TCP;
	t.n_psize = (uint32_t)iplen;
	t.n_bytes = (uint32_t)(dlen - tp->th_off * 4) & 2047L;

	t.p_port = min(tp->th_sport, tp->th_dport);
	if (t.p_port < IPPORT_RESERVED || 
		(IS_USER_PORT(tp->th_sport) && IS_USER_PORT(tp->th_dport))) {
		t.who_srv = tp->th_sport < tp->th_dport ? 1 : 2;
	}
	else if (tp->th_sport == tp->th_dport) {
		t.who_srv = 3;
	}
	/* tp->th_sport > IPPORT_RESERVED && tp->th_dport > IPPORT_RESERVED */
	else {
		if (CHECK_SRV(tp->th_sport, IPPROTO_TCP)) {
			t.who_srv = 1;
		}
		else if (CHECK_SRV(tp->th_dport, IPPROTO_TCP)) {
			t.who_srv = 2;
		}
		else {
			t.who_srv = !IS_USER_PORT(tp->th_sport) ? 1 : 2 ;
		}
		t.p_port = (t.who_srv == 2 ? tp->th_dport : tp->th_sport);
	}

	traf_add(&t);

	HTONS(tp->th_sport);
	HTONS(tp->th_dport);
	return;
}

/*
 * Calculate data length and server port in udp packets.
 */
void
traf_udp(up, ip)
	struct udphdr *up;
	struct ip *ip;
{
	u_char *cp = (u_char *)(up + 1);
	struct t_entry t;

	if (cp > snapend || dlen < sizeof(struct udphdr))
		return;

	NTOHS(up->uh_sport);
	NTOHS(up->uh_dport);
	NTOHS(up->uh_ulen);

	t.in_ip.s_addr = ip->ip_src.s_addr;
	t.out_ip.s_addr = ip->ip_dst.s_addr;
	t.ip_protocol = IPPROTO_UDP;
	t.n_psize = (uint32_t)iplen;
	t.n_bytes = (uint32_t)(up->uh_ulen - sizeof(*up)) & 2047L;

	t.p_port = min(up->uh_sport, up->uh_dport);
	if (t.p_port < IPPORT_RESERVED || 
		(IS_USER_PORT(up->uh_sport) && IS_USER_PORT(up->uh_dport))) {
		t.who_srv = up->uh_sport < up->uh_dport ? 1 : 2;
	}
	else if (up->uh_sport == up->uh_dport) {
		t.who_srv = 3;
	}
	/* tp->th_sport > IPPORT_RESERVED && tp->th_dport > IPPORT_RESERVED */
	else {
		if (CHECK_SRV(up->uh_sport, IPPROTO_TCP)) {
			t.who_srv = 1;
		}
		else if (CHECK_SRV(up->uh_dport, IPPROTO_TCP)) {
			t.who_srv = 2;
		}
		else {
			t.who_srv = !IS_USER_PORT(up->uh_sport) ? 1 : 2 ;
		}
		t.p_port = (t.who_srv == 2 ? up->uh_dport : up->uh_sport);
	}

	traf_add(&t);

	HTONS(up->uh_sport);
	HTONS(up->uh_dport);
	HTONS(up->uh_ulen);
	return;
}

/*
 * All other ip packets also collect.
 */
void
traf_ip(ip)
	register struct ip *ip;
{
	struct t_entry t;

	t.in_ip.s_addr = ip->ip_src.s_addr;
	t.out_ip.s_addr = ip->ip_dst.s_addr;
	t.ip_protocol = ip->ip_p;
	t.n_psize = (uint32_t)iplen;
	t.n_bytes = (uint32_t)dlen;
	t.p_port = t.who_srv = 0;
	traf_add(&t);
	return;
}

/*
 * Drop data to disk.
 * If mode = "w" then dump full table, else drop only essential data.
 */
void
traf_save(fname, mode)
	char *fname;
	char *mode;
{
	register FILE *fd;
	register int i;
	struct timeval tv;
	struct tv32 now;
	int32_t c = 0, ess_entry = 0;
	char *msg;

	if ((fd = fopen(fname, mode)) == NULL) {
		(void)syslog(LOG_WARNING, "(%s) fopen %s: %m",
			     device_name, fname);
		return;
	}
	gettimeofday(&tv, (struct timezone *)NULL);
	TV2TV32(&tv, &now);
	if (*mode == 'w') {
		int32_t max = MAX_T_ENTRIES;

		c += fwrite(&max, sizeof(max), 1, fd);
		c += fwrite(&n_entry, sizeof(n_entry), 1, fd);
		ess_entry = n_entry;
		msg = "dumped";
	} else {
		for (i = 0; i < MAX_T_ENTRIES; i++)
			if (entries[i].n_psize > 0 &&
				entries[i].n_psize >= save_min_record_size)
					ess_entry++;
		c += fwrite((int *)&ess_entry, sizeof(ess_entry), 1, fd);
		msg = "append";
	}
	c += fwrite(&begin_time, sizeof(begin_time), 1, fd);
	c += fwrite(&now, sizeof(now), 1, fd);
	if (*mode == 'w') {
		c += fwrite((struct t_entry *)entries, sizeof(struct t_entry),
			    MAX_T_ENTRIES, fd);
		(void)fwrite(&c, sizeof(c), 1, fd);
	} else
		for (i = 0; i < MAX_T_ENTRIES; i++) {
			if (entries[i].n_psize > 0 &&
				entries[i].n_psize >= save_min_record_size)
			    	fwrite((struct t_entry *)&entries[i],
			        	sizeof(struct t_entry), 1, fd);
		}
	(void)fclose(fd);
	(void)syslog(LOG_INFO, "(%s) %d records %s to %s", device_name,
		     ess_entry, msg, fname);
	return;
}

/*
 * Restore data from disk into memory. Useful when need to resume collect,
 * for example, after shutdown.
 */
void
traf_restore()
{
	FILE *fd;
	struct timeval tv;
	struct tv32 now;
	int32_t max_e, count_e, ncount;
	int rcount = 0;

	if ((fd = fopen(file_out, "r")) == NULL) {
		(void)syslog(LOG_WARNING, "(%s) can't restore hash table from %s: %m",
			     device_name, file_out);
		gettimeofday(&tv, (struct timezone *)NULL);
		TV2TV32(&tv, &begin_time);
		return;
	}
	rcount += fread(&max_e, sizeof(max_e), 1, fd);
	rcount += fread(&count_e, sizeof(count_e), 1, fd);
	rcount += fread(&begin_time, sizeof(begin_time), 1, fd);
	rcount += fread(&now, sizeof(now), 1, fd);
	/*
	 * if the dump was done by trafd with same MAX_T_ENTRIES,
	 * than we just read entire entries table,
	 * else we should reinsert all entries into the hash
	 * table one by one.
	 */
	if (max_e == MAX_T_ENTRIES) {
		rcount += fread((struct t_entry *)entries, sizeof(struct t_entry),
			   MAX_T_ENTRIES, fd);
		n_entry = count_e;
	} else {
		struct t_entry te;
		int ii;

		for (ii = 0; ii < max_e; ii++) {
			if (fread(&te, sizeof(struct t_entry), 1, fd) != 1) {
				(void)syslog(LOG_WARNING,
					(feof(fd) ?
					 	"(%s) can't restore from %s: corrupt table" :
					 	"(%s) can't restore from %s: %m"),
					device_name, file_out);
				(void)traf_clear();
				unlink(file_out);
				(void)fclose(fd);
				return;
			}
			if (te.n_psize)
				traf_add(&te);
			rcount++;
		}
	}
	(void)fread(&ncount, sizeof(ncount), 1, fd);
	unlink(file_out);
	(void)fclose(fd);
	if (rcount != ncount) {
		(void)syslog(LOG_WARNING, "(%s) can't restore from %s: corrupt table",
		     device_name, file_out);
		(void)traf_clear();
	} else {
		(void)syslog(LOG_NOTICE, "(%s) %d records restored from %s",
			     device_name, n_entry, file_out);
	}
}

/*
 * Full clear hash table, start time and misc counters. 
 */
void
traf_clear()
{
	struct timeval tv;
	(void)bzero((struct t_entry *)entries,
		    sizeof(struct t_entry) * MAX_T_ENTRIES);
	gettimeofday(&tv, (struct timezone *)NULL);
	TV2TV32(&tv, &begin_time);
	(void)syslog(LOG_INFO, "(%s) %d records cleared",
		     device_name, n_entry);
	n_entry = 0;
	return;
}

/* write data to fd */
int
traf_send(int fd)
{
	int ii;
	struct timeval tv;
	struct tv32 now;
	int32_t count, ess_entry = 0;
	int rc;

	/* calculate number of non-empty records */
	for (ii = 0; ii < MAX_T_ENTRIES; ii++)
		if (entries[ii].n_psize > 0 &&
			entries[ii].n_psize >= pipe_min_record_size)
				ess_entry++;

	gettimeofday(&tv, (struct timezone *)NULL);
	TV2TV32(&tv, &now);
	rc = write(fd, &ess_entry, sizeof(ess_entry));
	if (rc == -1)
		return -1;
	count = rc;
	rc = write(fd, &begin_time, sizeof(begin_time));
	if (rc == -1)
		return -1;
	count += rc;
	rc = write(fd, &now, sizeof(now));
	if (rc == -1)
		return -1;
	count += rc;
	for (ii = 0; ii < MAX_T_ENTRIES; ii++) {
		/* send only non-empty records */
		if (entries[ii].n_psize > 0 &&
			entries[ii].n_psize >= pipe_min_record_size)
		{
			rc = write(fd, (struct t_entry *)&entries[ii],
		    	sizeof(struct t_entry));
			if (rc == -1)
				return -1;
			count += rc;
		}
	}
	rc = write(fd, &count, sizeof(count));
	if (rc == -1)
		return -1;
	return ess_entry;
}

/*
 * This point will be called from pcap_dispatch() per each packet.
 */
void
processing_ip(bp, length)
	const uint8_t *bp;
	u_int length;
{
	struct ip *ip = (struct ip*) bp;
	int hlen;
	unsigned char *cp;

	if ((uint8_t *)(ip + 1) > snapend)
		return;
	if (length < sizeof(struct ip)) {
		syslog(LOG_WARNING, "(%s) truncated-ip: discard %d bytes",
		       device_name, length);
		return;
	}
	iplen = (int)ntohs(ip->ip_len);
	if (length < iplen)
		syslog(LOG_INFO, "(%s) truncated-ip: %d bytes missing",
			device_name, iplen - length);
	hlen = ip->ip_hl * 4;
	dlen = iplen - hlen;
	/*
	 * If this fragment zero, hand it to the next higher level protocol,
	 * else it is fragmented datagram.
	 */
	if (!(ntohs(ip->ip_off) & 0x1fff)) {
		cp = (unsigned char *)ip + hlen;
		switch (ip->ip_p) {
		case IPPROTO_TCP:
			traf_tcp((struct tcphdr *)cp, ip);
			break;
		case IPPROTO_UDP:
			traf_udp((struct udphdr *)cp, ip);
			break;
		default:
			traf_ip(ip);
			break;
		}
	} else	traf_ip(ip);
}

/* The END */
