/*-
 * Copyright (c) 2006 Stanislav A. Svirid. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *  This product includes software developed by the University of
 *  California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $RISS$
 *
 */

#ifndef _TRAFDB_H
#define _TRAFDB_H

#include <sys/types.h>

#include <netinet/in.h>

#include <limits.h>
#include <time.h>

#include "config.h"

#define TRAFDB_CMD_LIST	    1
#define TRAFDB_CMD_COPY	    2
#define TRAFDB_CMD_REPLACE  3
#define TRAFDB_CMD_UNKNOWN  -1

struct old_record {
	struct in_addr	in_ip;
	struct in_addr	out_ip;
	uint8_t	ip_protocol;
	uint8_t	who_srv;
	uint16_t	p_port;
	uint32_t	n_psize;
	uint32_t	n_bytes;
};

typedef union {
	struct old_record   old;
} tdb_record;
typedef tdb_record *tdb_record_t;

typedef struct trafdb_entry {
	int	records;
	time_t	begin;
	time_t	end;
	int	record_size;
	void	*data;
} tdb_entry;
typedef struct trafdb_entry *tdb_entry_t;

#define TRAFDB_SUMMARY	1
#define TRAFDB_APPEND	2
#define TRAFDB_COMPAT	4

struct trafdb {
	uint8_t	flags;	    /* Process DB flags */
	FILE	*_in;	    /* input database descriptor */
	FILE	*_out;	    /* output database descriptor */
	char	in_name[PATH_MAX];
	char	out_name[PATH_MAX];
	time_t	begin;	    /* Date or number first entry to process */
	time_t	end;	    /* Date or number last entry to process */
	int	entry;	    /* Current entry number */
	tdb_entry   last;   /* Last readed entry */
};
typedef struct trafdb	*trafdb_t;

#define is_sum(db) (db->flags & TRAFDB_SUMMARY)
#define is_append(db) (db->flags & TRAFDB_APPEND)
#define is_compat(db) (db->flags & TRAFDB_COMPAT)

void	trafdb_init(const char *, const char *, trafdb_t);
void	trafdb_close(trafdb_t);
int	trafdb_get_header(trafdb_t);
void	trafdb_skip_records(trafdb_t);
void	trafdb_load_records(trafdb_t);
tdb_record_t trafdb_get_record(trafdb_t, int);

void	trafdb_list_entries(trafdb_t);
void	trafdb_copy_entries(trafdb_t);
void	trafdb_replace(trafdb_t);

#endif /* !_TRAFDB_H */
