/*
 *	Copyright (c) 1999 RISS-Telecom Networking Center
 *
 *	Copyright (c) 1993 The CAD lab of the
 *	Novosibirsk Institute of Broadcasting and Telecommunication
 *
 *	BPFT $Id: traflog.c,v 1.6 2008/02/21 07:20:30 lorf Exp $
 *
 *	$Log: traflog.c,v $
 *	Revision 1.6  2008/02/21 07:20:30  lorf
 *	- Compile with large file offset support under Linux
 *	- Fix Linux compatibility
 *	- Corrections for GCC 4
 *	
 *	Revision 1.5  2006/11/01 13:07:39  frol
 *	 o Fixed database size > 2 GB in traflog and lib/trafdb.c
 *	
 *	Revision 1.2  2000/07/11 06:46:34  count
 *	      (Vitaly)
 *	
 *	Revision 1.1.1.1  2000/04/07 05:26:48  count
 *	Initial revision
 *	
 * Revision 1.6  1994/01/14  11:36:44  bob
 * Bug fixed in display tcp/udp fragmented datagramms
 *
 * Revision 1.5  1994/01/06  15:31:54  bob
 * Support new format of daemon log files
 *
 * Revision 1.4  1993/11/18  14:53:06  bob
 * Fixed bug with fseek length
 *
 * Revision 1.3  1993/11/06  12:58:37  bob
 * Change default output on last record, add '-a' for all records
 *
 * Revision 1.2  1993/11/01  15:13:30  bob
 * Now support '-r' flag for output only number of kb or number of records
 *
 * Revision 1.1  1993/10/20  16:05:18  bob
 * Initial revision
 *
 *
 * Redistribution and use in source forms, with and without modification,
 * are permitted provided that this entire comment appears intact.
 * Redistribution in binary form may occur without any restrictions.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' WITHOUT ANY WARRANTIES OF ANY KIND.
 */

/*	traflog.c - tcp/udp data traffic log manager	*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/param.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pcap.h>

#include "interface.h"
#include "addrtoname.h"
#include "traffic.h"
#include "util.h"

#include "traflog.h"

int range(register struct t_header *);
void traf_write(FILE *);
void traf_print(void);

static struct pat_mask *pat = NULL;
static struct t_header sumh;
static struct t_entry *sum = NULL;
static unsigned long sum_alloced_entries;
static unsigned long no = 0;
static int n_call = 0;
static int n_pat = 0;

uint32_t
addrmask(addr)
	uint32_t addr;
{
	register uint32_t m = INADDR_NONE;

	if (addr)
		while (!(addr & IN_CLASSA_NET))
			addr <<= IN_CLASSC_NSHIFT, m >>= IN_CLASSC_NSHIFT;
	else
		m = INADDR_ANY;

	return m;
}

char *
hostnetstr(in, mask)
	struct in_addr in;
	uint32_t mask;
{
	char *cp = NULL;
	static char line[MAXHOSTNAMELEN + 1];
	register uint32_t i;

	i = ntohl(in.s_addr);
	NTOHL(mask);
	i &= mask;
	if (!i) {
		strlcpy(line, "all", sizeof(line));
		return line;
	}
	if (!(htonl(i) ^ in.s_addr) && addrmask(in.s_addr) == INADDR_NONE) {
		strlcpy(line, ipaddr_string(&in), sizeof(line));
		return line;
	}
	if (!nflag) {
		register uint32_t net = i;
		struct netent *np = NULL;

		while ((mask & 1) == 0)
			mask >>= 1, net >>= 1;
		if ((np = getnetbyaddr(net, AF_INET)))
			cp = np->n_name;
	}	
#define	C(x)	((int)((x) & 0xff))
	if (cp)
		strlcpy(line, cp, sizeof(line));
	else if ((i & 0xffffff) == 0)
		snprintf(line, sizeof(line), "%u", C(i >> 24));
	else if ((i & 0xffff) == 0)
		snprintf(line, sizeof(line), "%u.%u", C(i >> 24) , C(i >> 16));
	else if ((i & 0xff) == 0)
		snprintf(line, sizeof(line), "%u.%u.%u", C(i >> 24),
			C(i >> 16), C(i >> 8));
	else
		snprintf(line, sizeof(line), "%u.%u.%u.%u", C(i >> 24),
			C(i >> 16), C(i >> 8), C(i));
#undef C
	return line;
}

int
pat_print(void)
{
	register int i;
	char port[16], proto[16];
	struct servent *svc;

	printf("%-24s %-24s %-10s %s\n", "From", "To", "Port", "Protocol");
	for (i = 0; i < n_pat; i++) {
		printf("%-24.24s ", hostnetstr(pat[i].in_ip,
					       pat[i].in_mask.s_addr));
		printf("%-24.24s ", hostnetstr(pat[i].out_ip,
					       pat[i].out_mask.s_addr));
		port[0] = proto[0] = 0;
		if (pat[i].p_port) {
			register u_short p = htons(pat[i].p_port);
			if (pat[i].ip_protocol != IPPROTO_UDP)
				if ((svc = getservbyport(p, "tcp"))) {
					strlcpy(port, svc->s_name, sizeof(port));
					strlcpy(proto, "tcp ", sizeof(proto));
				}
			if (pat[i].ip_protocol != IPPROTO_TCP)
				if ((svc = getservbyport(p, "udp"))) {
					if (!port[0])
						strlcpy(port, svc->s_name, sizeof(port));
					strlcpy(proto, "udp", sizeof(proto));
				}
			if (port[0] && !nflag)
				printf("%-10.10s ", port);
			else
				printf("%-10d ", pat[i].p_port);
		} else {
			printf("%-10.10s ", "all");
			if (pat[i].ip_protocol == IPPROTO_TCP)
				strlcpy(proto, "tcp", sizeof(port));
			if (pat[i].ip_protocol == IPPROTO_UDP)
				strlcpy(proto, "udp", sizeof(proto));
		}
		printf("%s\n", proto[0] ? proto : "tcp udp");
	}
	return 0;
}

uint32_t
strhostnet(cmdarg, mask)
	char *cmdarg;
	uint32_t *mask;
{
	struct in_addr in;
	struct netent *np;
	struct hostent *hp;

	if ((in.s_addr = inet_network(cmdarg)) != INADDR_NONE) {
		net_mask((uint32_t *)&in.s_addr);
		HTONL(in.s_addr);
		*mask = addrmask(in.s_addr);
		return in.s_addr;
	}
	if (!strcasecmp(cmdarg, "all")) {
		*mask = INADDR_ANY;
		return INADDR_ANY;
	}
	if ((np = getnetbyname(cmdarg))) {
		in.s_addr = np->n_net;
		*mask = htonl(net_mask((uint32_t *)&in.s_addr));
		return htonl(in.s_addr);
	}
	if ((hp = gethostbyname(cmdarg))) {
		in.s_addr = *(uint32_t *)*hp->h_addr_list;
		*mask = addrmask(in.s_addr);
		return in.s_addr;
	}
	errorx("unknown host or network '%s'", cmdarg);
	return 0;
}

int
iscmd(token)
	register char *token;
{
	if (!strcasecmp(token, "from"))
		return FROM;
	if (!strcasecmp(token, "to"))
		return TO;
	if (!strcasecmp(token, "port"))
		return PORT;
	if (!strcasecmp(token, "proto"))
		return PROTO;
	if (!strcasecmp(token, "mask"))
		return MASK;
	return NOCMD;
}

void
alloc_pat()
{
	if ((pat = realloc(pat, ++n_pat * sizeof(struct pat_mask))) == NULL)
		error("realloc");
	if (n_pat > 1)
		bcopy(pat + n_pat-2, pat + n_pat-1, sizeof(struct pat_mask));
	else
		bzero(pat, sizeof(struct pat_mask));
}

void
pattern(cmdbuf)
	char *cmdbuf;
{
	register int cmd, cmdp = MASK;
	register char *cmdarg;
	char *token;
	struct servent *svc;

	if (cmdbuf)
	while ((token = strtok(cmdbuf, " \t\r\n"))) {
		if (*token == '#' || *token == ';')
			continue;
		if ((cmd = iscmd(token)) == NOCMD)
			errorx("unknown keyword '%s'", token);
		if (!(cmdarg = strtok(NULL, " \t\r\n")))
			errorx("keyword '%s': missing argument", token);
		if (cmd <= cmdp)
			alloc_pat();
		switch (cmd) {
		case FROM:
			pat[n_pat-1].in_ip.s_addr = strhostnet(cmdarg,
						&pat[n_pat-1].in_mask.s_addr);
			break;
		case TO:
			pat[n_pat-1].out_ip.s_addr = strhostnet(cmdarg,
						&pat[n_pat-1].out_mask.s_addr);
			break;
		case MASK:
			switch(cmdp) {
			case FROM:
				pat[n_pat-1].in_mask.s_addr = inet_addr(cmdarg);
				break;
			case TO:
				pat[n_pat-1].out_mask.s_addr = inet_addr(cmdarg);
				break;
			default:
				errorx("inconsistent usage '%s %s'", token, cmdarg);
			}
			cmd = cmdp;
			break;
		case PORT:
			if (!(pat[n_pat-1].p_port = (u_short)atoi(cmdarg))) {
				if (!strcasecmp(cmdarg, "all")) {
					pat[n_pat-1].p_port = 0;
					break;
				}
				if ((svc = getservbyname(cmdarg, "tcp")))
					pat[n_pat-1].p_port = ntohs(svc->s_port);
				if ((svc = getservbyname(cmdarg, "udp")))
					pat[n_pat-1].p_port = ntohs(svc->s_port);
				if (!pat[n_pat-1].p_port)
					errorx("unknown port '%s'", cmdarg);
			}
			break;
		case PROTO:
			if (!strcasecmp(cmdarg, "udp")) {
				pat[n_pat-1].ip_protocol = IPPROTO_UDP;
				break;
			}
			if (!strcasecmp(cmdarg, "tcp")) {
				pat[n_pat-1].ip_protocol = IPPROTO_TCP;
				break;
			}
			errorx("unknown protocol '%s'", cmdarg);
		default:
			errorx("unknown keyword '%s'", token);
		}
		cmdp = cmd;
		cmdbuf = NULL;
	}
	if (!n_pat)
		alloc_pat();
	return;
}

int
getheader(fd, header)
	register FILE *fd;
	register struct t_header *header;
{
	if (fread((struct t_header *)header, sizeof(struct t_header), 1, fd) != 1) {
		if (!feof(fd))
			warning("fread: can't read header");
		return 0;
	}
	/*if (header->t_size > MAX_TO_SAVE) {
		warningx("table too big to fit into memory");
		return 0;
	}*/
	if (!header->t_size)
		warningx("empty table");
	return 1;
}

/*
 * Output contents list of daemon logfile.
 */
int
contents(fd, h)
	register FILE *fd;
	register struct t_header *h;
{
	register int i;
	register uint64_t bytes = 0, psize = 0;
	struct t_entry t;
	time_t start_clock, stop_clock;

	if (!n_call && !rflag)
		printf("\
 #        Started            Dumped            Data         All     Recs\n");

	for (i = 0; i < h->t_size; i++) {
		if (fread((struct t_entry *)&t, sizeof(struct t_entry), 1, fd) != 1) {
			warning("fread: can't read entry");
			return 0;
		}
		bytes += t.n_bytes, psize += t.n_psize;
	}
	if (!rflag) {
		start_clock = (time_t)h->start.tv32_sec;
		stop_clock = (time_t)h->stop.tv32_sec;
		printf("%03lu   %.15s", no, ctime(&start_clock) + 4);
		printf("%18.15s   %10" PRINTF_U64 "   %10" PRINTF_U64 "   %4d\n",
				ctime(&stop_clock)+4, bytes, psize, h->t_size);
	}
	return 1;
}

/*
 * print header
 */
void
print_header(h)
	register struct t_header *h;
{
	time_t start_clock, stop_clock;

	start_clock = h->start.tv32_sec;
	stop_clock = h->stop.tv32_sec;
	printf("%03lu   %.15s", no, ctime(&start_clock) + 4);
	printf("%18.15s %4d\n", ctime(&stop_clock) + 4, h->t_size);
}

int
patmatch(e)
	register struct t_entry *e;
{
	register int i = 0;

	if (e->n_psize == 0)
		return 0;
	for (i = 0; i < n_pat; i++) {
		if ((e->in_ip.s_addr & pat[i].in_mask.s_addr) ^
		      pat[i].in_ip.s_addr)
			continue;
		if ((e->out_ip.s_addr & pat[i].out_mask.s_addr) ^
		    pat[i].out_ip.s_addr)
			continue;
		if (pat[i].ip_protocol && e->ip_protocol != pat[i].ip_protocol)
			continue;
		if (!pat[i].p_port || e->p_port == pat[i].p_port)
			break;
	}
	return (n_pat - i);
}

int
ioblock(fd, th)
	FILE *fd;
	struct t_header *th;
{
	int i, j;
	struct t_entry te;

	if (!sflag || !n_call) {
		bzero(&sumh, sizeof(sumh));
		if (sum != NULL)
			free(sum);
		sum = NULL;
		sum_alloced_entries = 0;
	}

	for (i = 0; i < th->t_size; i++) {
		if (fread(&te, sizeof(te), 1, fd) != 1) {
			warning("fread: can't read table");
			return 0;
		}
		if (!patmatch(&te))
			continue;
		if (sflag && n_call) {
			for (j = 0; j < sumh.t_size; j++) {
				if (te.in_ip.s_addr == sum[j].in_ip.s_addr &&
				    te.out_ip.s_addr == sum[j].out_ip.s_addr &&
				    te.ip_protocol == sum[j].ip_protocol &&
				    te.who_srv == sum[j].who_srv &&
				    te.p_port == sum[j].p_port)
				{
					sum[j].n_bytes += te.n_bytes;
					sum[j].n_psize += te.n_psize;
					break;
				}
			}

			if (j < sumh.t_size)
				continue;
		}

		sumh.t_size++;
		if (sumh.t_size > sum_alloced_entries) {
			unsigned long newalloc;

			if (sum_alloced_entries == 0)
				newalloc = 4096 / sizeof(struct t_entry);
			else
				newalloc = sum_alloced_entries * 2;
			sum = (struct t_entry *) realloc(sum,
				newalloc * sizeof(struct t_entry));
			if (sum == NULL) {
				warning("realloc");
				return 0;
			}

			sum_alloced_entries = newalloc;
		}

		/*sum = (struct t_entry *) realloc(sum, ++sumh.t_size *
			sizeof(struct t_entry));
		if (sum == NULL) {
			warning("realloc");
			return 0;
		}*/
		memcpy(&sum[sumh.t_size - 1], &te, sizeof(struct t_entry));
	}
	if (!sumh.start.tv32_sec) {
		sumh.start.tv32_sec = th->start.tv32_sec;
		sumh.start.tv32_usec = th->start.tv32_usec;
	}
	sumh.stop.tv32_sec = th->stop.tv32_sec;
	sumh.stop.tv32_usec = th->stop.tv32_usec;
	return 1;
}

void
log_loop(fd, fdw)
	FILE *fd;
	FILE *fdw;
{
	struct t_header h;
	int retval;
	off_t last_pos = 0;

	sum_alloced_entries = 0;

	while (getheader(fd, &h)) {
		no++;
		last_pos = ftello(fd);

		if (range(&h)) {
			if (lflag)
				retval = contents(fd, &h);
			else if (aflag || sflag)
				retval = ioblock(fd, &h);
			else
				retval = !fseeko(fd, h.t_size * sizeof(struct t_entry), SEEK_CUR);
			if (retval) n_call++;
			else break;
			if (aflag && !sflag) {
				if (fdw) traf_write(fdw);
				else traf_print();
			}
		} else
			fseeko(fd, h.t_size * sizeof(struct t_entry), SEEK_CUR);
	}
	if (aflag && !sflag)
		return;
	if (lflag) {
		if (rflag)
			printf("%d\n", n_call);
		return;
	}

	if (last_pos && !sflag) {
		fseeko(fd, last_pos - sizeof(h), SEEK_SET);
		getheader(fd, &h);
		ioblock(fd, &h);
	}

	/* Reduce allocated space to the actual size. */
	if (sumh.t_size && sumh.t_size < sum_alloced_entries) {
		sum = (struct t_entry *) realloc(sum,
			sumh.t_size * sizeof(struct t_entry));
	}

	if (fdw) traf_write(fdw);
	else traf_print();

	return;
}

int
range(h)
	struct t_header *h;
{
	int val = 0;

	if (tbflag) {
		if (tb.tv32_sec <= h->start.tv32_sec)
			++val;
	} else {
		if (begin <= no)
			++val;
	}
	if (teflag) {
		if (te.tv32_sec >= h->stop.tv32_sec)
			++val;
	} else {
		if (endin >= no)
			++val;
	}
	return (val == 2);
}

void
traf_write(fd)
	FILE *fd;
{
	if (fwrite((struct t_header *)&sumh, sizeof(struct t_header), 1, fd) != 1)
		error("fwrite header");
	if (fwrite((struct t_entry *)sum, sizeof(struct t_entry), sumh.t_size, fd) != sumh.t_size)
		error("fwrite table");
	return;
}

/*
int
sortbyfrom(left, right)
	register struct t_entry *left;
	register struct t_entry *right;
{
	if (htonl(left->in_ip.s_addr) < htonl(right->in_ip.s_addr))
		return LESS;
	if (htonl(left->in_ip.s_addr) > htonl(right->in_ip.s_addr))
		return GREATER;
	return EQUAL;
}

int
sortbyto(left, right)
	register struct t_entry *left;
	register struct t_entry *right;
{
	if (htonl(left->out_ip.s_addr) < htonl(right->out_ip.s_addr))
		return LESS;
	if (htonl(left->out_ip.s_addr) > htonl(right->out_ip.s_addr))
		return GREATER;
	return EQUAL;
}

int
sortbyport(left, right)
	register struct t_entry *left;
	register struct t_entry *right;
{
	if (htons(left->p_port) < htons(right->p_port))
		return LESS;
	if (htons(left->p_port) > htons(right->p_port))
		return GREATER;
	return EQUAL;
}
*/

int
sortbysize(left, right)
	register struct t_entry *left;
	register struct t_entry *right;
{
	if (left->n_psize < right->n_psize)
		return GREATER;
	if (left->n_psize > right->n_psize)
		return LESS;
	return EQUAL;
}

void
traf_print(void)
{
	register int i, j;
	uint64_t abytes = 0, dbytes = 0;
	char buf[MAXHOSTNAMELEN + 1], *port, *user, *proto;
	time_t start_clock, stop_clock;

	for (i = 0; i < sumh.t_size; i++)
		abytes += sum[i].n_psize, dbytes += sum[i].n_bytes;
	if (rflag) {
		printf("%" PRINTF_U64 "\n", abytes);
		return;
	}
	qsort(sum, sumh.t_size, sizeof(struct t_entry), sortbysize);
	if (!fvnum) {
		gethostname(buf, MAXHOSTNAMELEN);
		start_clock = (time_t)sumh.start.tv32_sec;
		stop_clock = (time_t)sumh.stop.tv32_sec;
		printf("\n (%s) %s at", device_name, buf);
		printf(" %.15s -", ctime(&start_clock) + 4);
		printf(" %.15s\n", ctime(&stop_clock) + 4);
		printf(" Summary: %" PRINTF_U64 " data bytes, %" PRINTF_U64
				" all bytes, %u records\n",
				dbytes, abytes, sumh.t_size);
		printf("\
     From           Port         To            Port  Proto     Data       All\n");
	}
	for (i = 0; i < sumh.t_size; i++) {
		port = "undef";
		switch(sum[i].ip_protocol) {
			case IPPROTO_TCP:
				if (sum[i].p_port)
					port = tcpport_string(sum[i].p_port);
				proto = "tcp";
				break;
			case IPPROTO_UDP:
				if (sum[i].p_port)
					port = udpport_string(sum[i].p_port);
				proto = "udp";
				break;
			case IPPROTO_ICMP:
				proto = "icmp";
				break;
			case IPPROTO_EGP:
				proto = "egp";
				break;
			case IPPROTO_OSPF:
				proto = "ospf";
				break;
			case IPPROTO_IGMP:
				proto = "igmp";
				break;
			default:
				proto = "unkn";
		}
		if (sum[i].p_port)	user = "client";
		else			user = "none";
		if (!fvnum) {
			printf("%-18.18s %-6.6s  ", ipaddr_string(&sum[i].in_ip),
			       (sum[i].who_srv & 1) ? port : user);
			printf("%-18.18s %-6.6s  ", ipaddr_string(&sum[i].out_ip),
			       (sum[i].who_srv & 2) ? port : user);
			printf("%-4.4s %9u %10u\n", proto, sum[i].n_bytes,
			       sum[i].n_psize);
		} else 
			for (j = 0; j < fvnum; j++)
				switch (fv[j].cmd) {
				case FROM:
					printf(fv[j].format, ipaddr_string(&sum[i].in_ip));
					break;
				case TO:
					printf(fv[j].format, ipaddr_string(&sum[i].out_ip));
					break;
				case SPORT:
					printf(fv[j].format,
					       (sum[i].who_srv & 1) ? port : user);
					break;
				case DPORT:
					printf(fv[j].format,
					       (sum[i].who_srv & 2) ? port : user);
					break;
				case PROTO:
					printf(fv[j].format, proto);
					break;
				case BYTES:
					printf(fv[j].format, sum[i].n_bytes);
					break;
				case PSIZE:
					printf(fv[j].format, sum[i].n_psize);
					break;
				case FTIME:
					start_clock = (time_t)sumh.start.tv32_sec;
					strftime(buf, sizeof(buf), fv[j].format,
						 localtime(&start_clock));
					printf("%s", buf);
					break;
				case LTIME:
					stop_clock = (time_t)sumh.stop.tv32_sec;
					strftime(buf, sizeof(buf), fv[j].format,
						 localtime(&stop_clock));
					printf("%s", buf);
					break;
				}
	}
}

