/*
 * Copyright (c) 2002 RISS-Telecom, Novosibirsk, Russia
 * $RISS: trafd/trafrev/trafrev.c,v 1.1 2004/11/17 09:21:02 count Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/param.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pcap.h>

#include "interface.h"
#include "addrtoname.h"
#include "traffic.h"
#include "util.h"

#include "trafrev.h"

int range(register struct t_header *);
int traf_write(FILE *, FILE *, struct t_header *);

static int no = 0;

int
getheader(fd, header)
	register FILE *fd;
	register struct t_header *header;
{
	if (fread((struct t_header *)header, sizeof(struct t_header), 1, fd) != 1 && feof(fd)) {
		warning("fread: can't read header");
		return 0;
	}
	return 1;
}

void
log_loop(fd, fdw)
	FILE *fd;
	FILE *fdw;
{
	struct t_header h;
	off_t pos;
	time_t start_clock, stop_clock;

	if (fdw == NULL)
		fdw = stdout;

#define POLL_INTERVAL	(60*60*3 + 5)

	if (fseeko(fd, 0, SEEK_END) == -1) {
		warning("fseeko");
		return;
	}
	pos = ftello(fd) - sizeof(h);

	while (pos >= 0) {
		if (fseeko(fd, pos, SEEK_SET) == -1) {
			warning("fseeko");
			return;
		}
		if (getheader(fd, &h) && h.stop.tv32_sec > h.start.tv32_sec &&
				(h.stop.tv32_sec - h.start.tv32_sec) <= POLL_INTERVAL &&
				range(&h) == 2) {
			start_clock = (time_t)h.start.tv32_sec;
			stop_clock = (time_t)h.stop.tv32_sec;
			fprintf(stderr, "%10" PRINTF_I64 "	 %.15s",
					pos, ctime(&start_clock) + 4);
			fprintf(stderr, "%18.15s %4u\n",
					ctime(&stop_clock) + 4, h.t_size);
			if(traf_write(fd, fdw, &h) != 0)
				return;
		}
		pos -= sizeof(struct t_entry);
	}

	return;
}

int
range(h)
	register struct t_header *h;
{
	register int val = 0;

	if (tbflag) {
		if (tb.tv32_sec <= h->start.tv32_sec)
			++val;
	} else {
		if (begin <= no)
			++val;
	}
	if (teflag) {
		if (te.tv32_sec >= h->stop.tv32_sec)
			++val;
	} else {
		if (endin >= no)
			++val;
	}
	return val;
}

int
traf_write(fd, fdw, h)
	FILE *fd;
	FILE *fdw;
	struct t_header *h;
{
	struct t_entry *block = NULL;
	int retval;

	retval = !fwrite(h, sizeof(struct t_header), 1, fdw);
			
	if (retval)
		error("fwrite header");

	if (h->t_size) {
		block = (struct t_entry *) malloc(h->t_size * sizeof(struct t_entry));
		if (block == NULL) {
			warning("alloca");
			return 1;
		}

		retval = fread(block, sizeof(struct t_entry), h->t_size, fd);
		if (retval != (size_t)h->t_size) {
			free(block);
			warning("fread: can't read table");
			return 1;
		}
			
		retval = fwrite(block, sizeof(struct t_entry), h->t_size, fdw);

		free(block);
		fflush(fdw);

		if (!retval)
			error("fwrite table");
	}
	
	return 0;
}

