/*
 * Copyright (c) 2002 RISS-Telecom, Novosibirsk, Russia
 *
 * $RISS: trafd/trafslice/main.c,v 1.2 2004/11/17 09:21:02 count Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <paths.h>
#include <fcntl.h>
#include <signal.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <pcap.h>

#include "interface.h"
#include "addrtoname.h"
#include "pathnames.h"
#include "util.h"
#include "traffic.h"

#include "trafslice.h"

void setthetime(register char *, struct tv32 *);

int aflag;		/* display all records, default last */
int fflag;		/* convert addresses to names only for local hosts */
int lflag;		/* output list of log file records */
int nflag;		/* don't convert addresses to host names */
int Nflag;		/* output host names without domain */
int rflag;		/* only return number of records or KB */
int sflag;		/* summary traffic */

#define	LASTOFFSET	999
int begin = 0, endin = LASTOFFSET;
int tbflag = 0, teflag = 0;
struct tv32 tb, te;

char *device_name = NULL;
char *f_end;

static struct timeval tval;

int fvnum = 0;
struct fmt_var *fv = NULL;

int
main(argc, argv)
	int argc;
	char **argv;
{
	register FILE *fd;
	FILE *fdw = NULL;
	register int op;
	char buf[256];
	char ebuf[PCAP_ERRBUF_SIZE];
	extern char *optarg;
	extern int opterr;
	bpf_u_int32 localnet, netmask;
	void usage(), onpipe();

	set_program_name(argv[0]);

	if (gettimeofday(&tval, NULL) == -1)
		error("time");

	opterr = 0;
	while ((op = getopt(argc, argv, "b:e:i:w:")) != EOF)
		switch (op) {
		case 'b':
			if (!(begin = atoi(optarg)))
				usage();
			if (begin > LASTOFFSET) {
				setthetime(optarg, &tb);
				++tbflag;
			}
			break;
		case 'e':
			if (!(endin = atoi(optarg)))
				usage();
			if (endin > LASTOFFSET) {
				setthetime(optarg, &te);
				++teflag;
			}
			break;
		case 'i':
			device_name = optarg;
			break;
		case 'w':
			if ((fdw = fopen(optarg, "w")) == NULL)
				error("can't open %s for writing", optarg);
			break;
		default:
			usage();
		}

	signal(SIGPIPE, onpipe);

	if (device_name == NULL)
		if ((device_name = getenv("IFF_LISTEN")) == NULL)
			if ((device_name = pcap_lookupdev(ebuf)) == NULL)
				errorx("%s", ebuf);

	if (pcap_lookupnet(device_name, &localnet, &netmask, ebuf) < 0) {
		localnet = 0;
		netmask = 0;
	}

	init_addrtoname(localnet, netmask);

	/* device_name is a path to statistics file */
	if (strchr(device_name, '/') != NULL)
		strlcpy(buf, device_name, sizeof(buf));
	else
		snprintf(buf, sizeof(buf), TRAFDDIR FMT_SAVEFILE, device_name);

	if ((fd = fopen(buf, "r")) == NULL)
		error("can't open %s for reading", buf);

	log_loop(fd, fdw);

	if (fdw)
		fclose(fdw);

	exit(0);
}

void
badformat(void)
{
	errorx("illegal time format");
}

static int days_in_month[12] =
	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	
#define	ATOI2(ar)	((ar)[0] - '0') * 10 + ((ar)[1] - '0'); (ar) += 2;

void
setthetime(p, tv)
	register char *p;
	struct tv32 *tv;
{
	register struct tm *lt;
	int dot;
	char *t;

	for (t = p, dot = 0; *t; ++t)
		if (!isdigit(*t) && (*t != '.' || dot++))
			badformat();

	lt = localtime(&tval.tv_sec);
	lt->tm_isdst = -1;

	if ((t = index(p, '.'))) {		/* .ss */
		*t++ = '\0';
		lt->tm_sec = ATOI2(t);
		if (lt->tm_sec >= 60)
			badformat();
	} else
		lt->tm_sec = 0;

	for (t = p; *t; ++t)
		if (!isdigit(*t))
			badformat();

	switch (strlen(p)) {
	case 10:				/* yy */
		lt->tm_year = ATOI2(p);
		if (lt->tm_year <= 69)		/* hack for 2000 ;-} */
			lt->tm_year += 100;
		/* FALLTHROUGH */
	case 8:					/* mm */
		lt->tm_mon = ATOI2(p);
		if (lt->tm_mon > 12)
			badformat();
		--lt->tm_mon;			/* time struct is 0 - 11 */
		/* FALLTHROUGH */
	case 6:					/* dd */
		lt->tm_mday = ATOI2(p);
		if (lt->tm_mday > days_in_month[lt->tm_mon] 
		    && !(((lt->tm_year % 4 == 0 && lt->tm_year % 100 != 0)
		    || lt->tm_year % 400 == 0) && lt->tm_mon == 1
		    && lt->tm_mday == 29))
			badformat();
		/* FALLTHROUGH */
	case 4:					/* hh */
		lt->tm_hour = ATOI2(p);
		if (lt->tm_hour > 23)
			badformat();
		/* FALLTHROUGH */
	case 2:					/* mm */
		lt->tm_min = ATOI2(p);
		if (lt->tm_min > 59)
			badformat();
		break;
	default:
		badformat();
	}

	/* convert broken-down time to GMT clock time */
	if ((tval.tv_sec = mktime(lt)) == -1)
		badformat();

	tv->tv32_sec = (int32_t)tval.tv_sec;
	tv->tv32_usec = 0;
}

void
onpipe()
{
	exit(1);
}

void
usage()
{
	extern char version[];

	fprintf(stderr, "traflog v%s - tcp/udp data traffic log manager\n", version);
	fprintf(stderr, "Usage: %s -i iface [-b #] [-e #] [-w file]\n",	program_name);
	exit(-1);
}
