/*
 * Copyright (c) 2002 RISS-Telecom, Novosibirsk, Russia
 * $RISS: trafd/trafslice/trafslice.c,v 1.4 2006/09/27 05:00:34 frol Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/param.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pcap.h>

#include "interface.h"
#include "addrtoname.h"
#include "traffic.h"
#include "util.h"

#include "trafslice.h"

int range(struct t_header *);
int traf_write(FILE *, FILE *, struct t_header *);

static int no = 0;

int
getheader(fd, header)
	FILE *fd;
	struct t_header *header;
{
	if (fread((struct t_header *)header, sizeof(struct t_header), 1, fd) != 1) {
		if (!feof(fd))
			warning("fread: can't read header");
		return 0;
	}
	return 1;
}

void
log_loop(fd, fdw)
	FILE *fd;
	FILE *fdw;
{
	struct t_header h;
#if 0
	off_t last_pos = 0;
	int got_it = 0;
#endif

	if (fdw == NULL)
		fdw = stdout;

	while (getheader(fd, &h)) {
		no++;
		if (h.t_size == 0) {
			warningx("empty table");
			continue;
		}

#if 0
		if ((last_pos = ftello(fd)) == -1) {
			warning("ftello");
			return;
		}
#endif

		if (range(&h) == 2) {
			if(traf_write(fd, fdw, &h) != 0)
				return;
#if 0
			got_it++;
#endif
		} else
			fseeko(fd, h.t_size * sizeof(struct t_entry), SEEK_CUR);
	}

#if 0
	if (!got_it && last_pos) {
		fseeko(fd, last_pos - sizeof(h), SEEK_SET);
		getheader(fd, &h);
		traf_write(fd, fdw, &h);
	}
#endif

	return;
}

int
range(h)
	struct t_header *h;
{
	int val = 0;

	if (tbflag) {
		if (tb.tv32_sec <= h->start.tv32_sec)
			++val;
	} else {
		if (begin <= no)
			++val;
	}
	if (teflag) {
		if (te.tv32_sec >= h->stop.tv32_sec)
			++val;
	} else {
		if (endin >= no)
			++val;
	}
	return val;
}

int
traf_write(fd, fdw, h)
	FILE *fd;
	FILE *fdw;
	struct t_header *h;
{
	struct t_entry *block = NULL;
	int retval;

	retval = !fwrite(h, sizeof(struct t_header), 1, fdw);

	if (retval)
		error("fwrite header");

	if (h->t_size) {
		block = (struct t_entry *) malloc(h->t_size * sizeof(struct t_entry));
		if (block == NULL) {
			warning("malloc");
			return 1;
		}

		retval = fread(block, sizeof(struct t_entry), h->t_size, fd);
		if (retval != h->t_size) {
			free(block);
			warning("fread: can't read table");
			return 1;
		}

		retval = !fwrite(block, sizeof(struct t_entry), h->t_size, fdw);

		free(block);

		if (retval)
			error("fwrite table");
	}

	return 0;
}

