/*
 *	Copyright (c) 1999 RISS-Telecom Networking Center
 *
 *	Copyright (c) 1993 The CAD lab of the
 *	Novosibirsk Institute of Broadcasting and Telecommunication
 *
 *	BPFT $Id: main.c,v 1.1.1.1 2004/11/17 09:06:31 count Exp $
 *
 *	$Log: main.c,v $
 *	Revision 1.1.1.1  2004/11/17 09:06:31  count
 *	Init import fo 3.0.3
 *	
 *	Revision 1.1.1.1  2000/04/07 05:26:48  count
 *	Initial revision
 *	
 * Revision 1.2  1993/10/30  10:15:30  bob
 * Modify access method to daemon
 *
 * Revision 1.1  1993/10/20  16:05:35  bob
 * Initial revision
 *
 *
 * Redistribution and use in source forms, with and without modification,
 * are permitted provided that this entire comment appears intact.
 * Redistribution in binary form may occur without any restrictions.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' WITHOUT ANY WARRANTIES OF ANY KIND.
 */

/*	main.c - tcp/udp data traffic daemon statistics	*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <paths.h>
#include <errno.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/un.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pcap.h>

#include "interface.h"
#include "addrtoname.h"
#include "pathnames.h"

#include "util.h"

#define	WAIT_DAEMON	5
#define	MAX_RETRY	5

/*
 * Command line switches.
 */
int bflag;	/* binary output */
int fflag;	/* convert addresses to names only for local hosts */
int nflag;	/* don't convert addresses to host names */
int Nflag;	/* output host names without domain */

char *device_name = 0;
char *host;

extern int errno;
extern int h_errno;
extern int traf_get();
extern void traf_put();
extern void traf_print();

int
main(argc, argv)
	int argc;
	char **argv;
{
	int ifd;
	pid_t pid;
	struct sockaddr_in server;
	struct sockaddr_un sun;
	struct hostent *hp;
	char buf[40];
	extern char *optarg;
	extern int optind, opterr;
	int opt;
	char ebuf[PCAP_ERRBUF_SIZE];
	bpf_u_int32 localnet, netmask;
	void cleanup(), /* onpipe(), */ onterm(), usage();
	u_short port;

	set_program_name(argv[0]);

	opterr = 0;
	while ((opt = getopt(argc, argv, "bfi:nN")) != EOF)
		switch (opt) {
		case 'b':
			++bflag;
			break;
		case 'f':
			++fflag;
			break;
		case 'i':
			device_name = optarg;
			break;
		case 'n':
			++nflag;
			break;
		case 'N':
			++Nflag;
			break;
		default:
			usage();
		}

	if (device_name == 0)
		if ((device_name = getenv("IFF_LISTEN")) == NULL)
			if ((device_name = pcap_lookupdev(ebuf)) == 0)
				errorx("%s", ebuf);
	
	if (pcap_lookupnet(device_name, &localnet, &netmask, ebuf) < 0) {
		localnet = 0;
		netmask = 0;
		warningx("%s", ebuf);
	}
	
	init_addrtoname(localnet, netmask);

	signal(SIGTERM, onterm);
	signal(SIGIO, SIG_IGN);
	signal(SIGURG, SIG_IGN);
	if (signal(SIGINT, SIG_IGN) != SIG_IGN)
		signal(SIGINT, cleanup);

	if ((host = argv[optind]) != NULL) {
		if (atoi(host) > 0) {	/* XXX lame */
			server.sin_family = AF_INET;
			server.sin_addr.s_addr = inet_addr(host);
		} else {
			if ((hp = gethostbyname(host)) == NULL) {
				snprintf(buf, sizeof(buf), "%s: gethostbyname",
					program_name);
				herror(buf);
				exit(1);
			}
			server.sin_family = hp->h_addrtype;
			bcopy(hp->h_addr, (char *)&server.sin_addr,
			      hp->h_length);
			host = hp->h_name;
		}
		++optind;
		port = argv[optind] ? atoi(argv[optind]) : TRAFSTAT_PORT;
		server.sin_port = htons(port);
		if ((ifd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
			error("socket AF_INET SOCK_STREAM");
		fprintf(stderr, "Trying %s:%d...\n", host, port);
		if (connect(ifd, (struct sockaddr *)&server, sizeof(server)) < 0)
			error("connect %s:%d", host, port);
		fprintf(stderr, "Connected to %s:%d\n", host, port);
		snprintf(buf, sizeof(buf), "-i %s -b\n", device_name);
		write(ifd, buf, strlen(buf));
	} else {
		ifd = socket(PF_LOCAL, SOCK_STREAM, 0);
		if (ifd == -1)
			error("socket PF_LOCAL SOCK_STREAM");

		sun.sun_family = AF_LOCAL;
		snprintf(sun.sun_path, sizeof(sun.sun_path), TRAFDDIR FMT_SOCKET,
			device_name);

		if (connect(ifd, (struct sockaddr *)&sun, sizeof(sun)) == -1) {
			close(ifd);
			error("connect %s", sun.sun_path);
		}
	}

	pid = traf_get(ifd);

	if (host != NULL)
		shutdown(ifd, 2);
	close(ifd);
	if (pid) {
		if (bflag)
			traf_put(1);
		else
			traf_print();
	} else
		error("can't obtain or empty traffic table");

	exit(0);
	/* NOTREACHED */
	return 0;
}

void
cleanup()
{
	fprintf(stderr, " ...%s: interrupted\n", program_name);
	exit(1);
}
/*
void
onpipe()
{
	fprintf(stderr, "%s: broken pipe\n", program_name);
	cleanup();
}
*/
void
onterm()
{
	fprintf(stderr, "%s: terminated\n", program_name);
	cleanup();
}

void
usage()
{
	extern char version[];

	fprintf(stderr,
"trafstat v%s - tcp/udp data traffic daemon statistics\n", version);
	fprintf(stderr,
"Usage: %s [-i iface] [-b | -fnN] [host] [port]\n", program_name);
	exit(-1);
}
