/*
 *	Copyright (c) 1999 RISS-Telecom Networking Center
 *	
 *	Copyright (c) 1993 The CAD lab of the
 *	Novosibirsk Institute of Broadcasting and Telecommunication
 *
 *	BPFT $Id: trafstatd.c,v 1.2 2008/02/21 07:20:30 lorf Exp $
 *
 *	$Log: trafstatd.c,v $
 *	Revision 1.2  2008/02/21 07:20:30  lorf
 *	- Compile with large file offset support under Linux
 *	- Fix Linux compatibility
 *	- Corrections for GCC 4
 *	
 *	Revision 1.1.1.1  2004/11/17 09:06:31  count
 *	Init import fo 3.0.3
 *	
 *	Revision 1.1.1.1  2000/04/07 05:26:48  count
 *	Initial revision
 *	
 * Revision 1.2  1993/11/01  12:07:15  bob
 * Extended log file records
 *
 * Revision 1.1  1993/10/20  16:06:02  bob
 * Initial revision
 *
 *
 * Redistribution and use in source forms, with and without modification,
 * are permitted provided that this entire comment appears intact.
 * Redistribution in binary form may occur without any restrictions.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' WITHOUT ANY WARRANTIES OF ANY KIND.
 */

/*	trafstatd.c - traffic statictics daemon, invoke only via inetd	*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <syslog.h>
#include <pcap.h>

#include "interface.h"
#include "pathnames.h"
#include "util.h"

#define	LINELEN		1024
#define	ENTRIES		50

int
main(int argc, char *argv[])
{
	char *lp;
	char **ap, *av[ENTRIES + 1], line[LINELEN + 1];
	struct sockaddr_in sin;
	socklen_t sval = sizeof sin;
#ifdef WITH_OLD_LOG
	struct hostent *hp;
	FILE *fplog;
#endif

	set_program_name(argv[0]);

	if (getpeername(0, (struct sockaddr *)&sin, &sval) < 0)
		error("getpeername");

	if (!fgets(line, sizeof(line), stdin))
		exit(1);

#ifndef WITH_OLD_LOG

	{
		char *lastp = &line[strlen(line)];
		char *type;

		if (lastp[-2] == 0xd) {
			lastp--;
			*(lastp-1) = '\n';
			*lastp = 0;
			type = "telnet";
		} else
			type = "trafstat";
		if (lastp[-1] != '\n')
			*lastp++ = '\n';

		openlog(program_name, LOG_PID, LOG_DAEMON);
		syslog(LOG_INFO, "%s: %s: %s", inet_ntoa(sin.sin_addr), type, line);
		closelog();
	}

#else /* WITH_OLD_LOG */

	/*
	 * PATH_TRAFSTAT_LOG must exist & be owned by whoever this process
	 * runs as defined inetd.conf for this fopen to succeed.
	 */
	if ((fplog = fopen(PATH_TRAFSTAT_LOG, "a")) != NULL) {
		struct timeval tp;
		char *lastp = &line[strlen(line)];

		gettimeofday(&tp, (struct timezone *)NULL);
		hp = gethostbyaddr((char *) &sin.sin_addr,
		     sizeof(sin.sin_addr), sin.sin_family);
		(void) fprintf(fplog, "%.15s %s: ", ctime(&tp.tv_sec) + 4,
			       (hp == NULL ? inet_ntoa(sin.sin_addr) : hp->h_name));
		if (lastp[-2] == 0xd) {
			lastp--;
			*(lastp-1) = '\n';
			*lastp = 0;
			fprintf(fplog, "telnet: ");
		} else
			fprintf(fplog, "trafstat: ");
		if (lastp[-1] != '\n')
			*lastp++ = '\n';
		fwrite(line, sizeof line[0], lastp - line, fplog);
		(void) fclose(fplog);
	}

#endif /* WITH_OLD_LOG */

	av[0] = "trafstat";
	for (lp = line, ap = &av[1];;) {
		*ap = strtok(lp, " \t\r\n");
		if (!*ap)
			break;
		if (++ap == av + ENTRIES)
			break;
		lp = NULL;
	}

	execv(PATH_TRAFSTAT, av);
	exit(1);
	/* NOTREACHED */
	return 0;
}
